/*---------------------------------------------------------------------------*\

  FILE........: version.h
  AUTHOR......: Tomas Härdin
  DATE CREATED: 03 November 2017

  Codec 2 VERSION #defines

\*---------------------------------------------------------------------------*/

/*
  Copyright (C) 2017 Tomas Härdin

  All rights reserved.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License version 2.1, as
  published by the Free Software Foundation.  This program is
  distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, see <http://www.gnu.org/licenses/>.
*/

//this functions both as an include guard and your typical HAVE macro
#ifndef CODEC2_HAVE_VERSION
#define CODEC2_HAVE_VERSION

#define CODEC2_VERSION_MAJOR 1
#define CODEC2_VERSION_MINOR 0
#define CODEC2_VERSION_PATCH 5
#define CODEC2_VERSION "1.0.5"

#endif //CODEC2_HAVE_VERSION
